local function GetLevelAndCPRequirementFromItemLink(itemLink)
  local link = { ZO_LinkHandler_ParseLink(itemLink) }
  return tonumber(link[5]), tonumber(link[6])
end

local function GetPotionPowerLevel(itemLink)
  local CP, level = GetLevelAndCPRequirementFromItemLink(itemLink)
  if level < 50 then
    return level
  end
  return CP
end

-- The index consists of the item's required level, required vet
-- level, quality, and trait(if any), separated by colons.
-- /script d(zo_strmatch("|H1:item:6000:30:1:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h", '|H.-:item:.-:(%d-)|h'))
local itemIndexCache = { }

function TTCCompanion:GetItemLinkParseData(itemLink)
  local itemType, specializedItemType = GetItemLinkItemType(itemLink)
  if itemType == ITEMTYPE_POISON or itemType == ITEMTYPE_POTION then
    return TTCCompanion:GetPotionEffectWritRewardField(itemLink)
  end
  if itemType == ITEMTYPE_MASTER_WRIT then
    return TTCCompanion:GetVoucherCountByItemLink(itemLink)
  end
  return 0
end

local function GetItemsTrait(itemLink, itemType)
  if itemType ~= ITEMTYPE_POISON and itemType ~= ITEMTYPE_POTION then
    return GetItemLinkTraitType(itemLink) or 0
  end
  local powerLevel = GetPotionPowerLevel(itemLink)
  return TTCCompanion.potionVarientTable[powerLevel] or 0
end

local function GetRequiredLevel(itemLink, itemType)
  return itemType ~= ITEMTYPE_RECIPE and GetItemLinkRequiredLevel(itemLink) or 1
end

local function CreateIndexFromLink(itemLink)
  local itemType, specializedItemType = GetItemLinkItemType(itemLink)
  return GetRequiredLevel(itemLink, itemType) .. ":" .. GetItemLinkRequiredChampionPoints(itemLink) / 10 .. ":" .. GetItemLinkDisplayQuality(itemLink) .. ":" .. GetItemsTrait(itemLink, itemType) .. ":" .. TTCCompanion:GetItemLinkParseData(itemLink)
end

-- /script d(LibGuildStore_Internal.GetOrCreateIndexFromLink("|H0:item:44714:308:50:0:0:0:0:0:0:0:0:0:0:0:0:36:0:0:0:0:853248|h|h"))
function TTCCompanion.GetOrCreateIndexFromLink(itemLink)
  local index = itemIndexCache[itemLink]
  if not index then
    itemIndexCache[itemLink] = CreateIndexFromLink(itemLink)
    index = itemIndexCache[itemLink]
  end
  return index
end

function TTCCompanion.GetItemLinkRecipeNumIngredients(itemLink)
  local numIngredients = GetItemLinkRecipeNumIngredients(itemLink)
  if numIngredients > 0 then
    return numIngredients
  end

  -- Clear player crafted flag and switch to H0 and see if this is an item resulting from a fixed recipe.
  local switchItemLink = string.gsub(string.gsub(itemLink, '0:1:0:0:0:0|h', '0:0:0:0:0:0|h'), '|H1:', '|H0:')
  if TTCCompanion.recipeData[switchItemLink] then
    return GetItemLinkRecipeNumIngredients(TTCCompanion.recipeData[switchItemLink])
  end


  --switch to MM pricing Item style
  local mmStyleLink = zo_strmatch(switchItemLink, '|H.-:item:(.-):')
  if mmStyleLink then
    mmStyleLink = mmStyleLink .. ':' .. TTCCompanion.GetOrCreateIndexFromLink(switchItemLink)
    if TTCCompanion.virtualRecipe[mmStyleLink] then
      return #TTCCompanion.virtualRecipe[mmStyleLink]
    end
  end
  return 0
end

function TTCCompanion.GetItemLinkRecipeIngredientInfo(itemLink, i)
  local ingredientItemLink = GetItemLinkRecipeIngredientItemLink(itemLink, i)
  if ingredientItemLink ~= '' then
    local _, _, numRequired = GetItemLinkRecipeIngredientInfo(itemLink, i)
    return ingredientItemLink, numRequired
  end

  local switchItemLink = string.gsub(string.gsub(itemLink, '0:1:0:0:0:0|h', '0:0:0:0:0:0|h'), '|H1:', '|H0:')
  if TTCCompanion.recipeData[switchItemLink] then
    return TTCCompanion.GetItemLinkRecipeIngredientInfo(TTCCompanion.recipeData[switchItemLink], i)
  end

  local mmStyleLink = zo_strmatch(switchItemLink, '|H.-:item:(.-):')
  if mmStyleLink then
    mmStyleLink = mmStyleLink .. ':' .. TTCCompanion.GetOrCreateIndexFromLink(switchItemLink)
    if TTCCompanion.virtualRecipe[mmStyleLink] then
      return TTCCompanion.virtualRecipe[mmStyleLink][i].item, TTCCompanion.virtualRecipe[mmStyleLink][i].required
    end
  end

  return nil, nil

end

function TTCCompanion:GetSolventItemLink(itemLink)
  local itemType, _ = GetItemLinkItemType(itemLink)
  local solventIndex = GetItemLinkRequiredLevel(itemLink) + GetItemLinkRequiredChampionPoints(itemLink)

  local solventItemLink = {
    [ITEMTYPE_POTION] = {
      [3] = "|H1:item:883:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h",
      [10] = "|H1:item:1187:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h",
      [20] = "|H1:item:4570:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h",
      [30] = "|H1:item:23265:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h",
      [40] = "|H1:item:23266:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h",
      [60] = "|H1:item:23267:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h",
      [100] = "|H1:item:23268:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h",
      [150] = "|H1:item:64500:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h",
      [200] = "|H1:item:64501:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h",
    },
    [ITEMTYPE_POISON] = {
      [3] = "|H1:item:75357:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h",
      [10] = "|H1:item:75358:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h",
      [20] = "|H1:item:75359:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h",
      [30] = "|H1:item:75360:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h",
      [40] = "|H1:item:75361:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h",
      [60] = "|H1:item:75362:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h",
      [100] = "|H1:item:75363:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h",
      [150] = "|H1:item:75364:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h",
      [200] = "|H1:item:75365:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h",
    },
  }
  return solventItemLink[itemType][solventIndex]
end

TTCCompanion.internalRecipeData = {
  [28281] = 17632,
  [28289] = 17626,
  [28293] = 17623,
  [28297] = 17620,
  [28299] = 17355,
  [28301] = 17617,
  [28304] = 17351,
  [28305] = 17614,
  [28308] = 17348,
  [28309] = 17611,
  [28311] = 17346,
  [28313] = 17608,
  [28316] = 17342,
  [28317] = 17605,
  [28320] = 17339,
  [28321] = 17566,
  [28325] = 17565,
  [28330] = 17563,
  [28331] = 17365,
  [28332] = 17380,
  [28333] = 17386,
  [28334] = 17560,
  [28338] = 17561,
  [28342] = 17558,
  [28346] = 17559,
  [28348] = 17288,
  [28350] = 17556,
  [28353] = 17284,
  [28354] = 17558,
  [28357] = 17281,
  [28358] = 27104,
  [28360] = 17279,
  [28362] = 17576,
  [28366] = 17573,
  [28370] = 17570,
  [28373] = 17302,
  [28374] = 17567,
  [28378] = 17564,
  [28382] = 17561,
  [28385] = 17293,
  [28386] = 17558,
  [28390] = 17555,
  [28394] = 17552,
  [28397] = 17284,
  [28398] = 17549,
  [28401] = 17569,
  [28402] = 17646,
  [28403] = 17217,
  [28405] = 17566,
  [28409] = 17563,
  [28411] = 17128,
  [28413] = 17560,
  [28415] = 17125,
  [28417] = 17557,
  [28419] = 17122,
  [28421] = 17554,
  [28423] = 17119,
  [28425] = 17551,
  [28427] = 17116,
  [28429] = 17548,
  [28431] = 17113,
  [28433] = 17545,
  [28435] = 17111,
  [28437] = 17542,
  [28439] = 17109,
  [28441] = 17555,
  [28443] = 17179,
  [28444] = 17606,
  [28445] = 17552,
  [28449] = 17549,
  [28452] = 17115,
  [28453] = 17546,
  [28456] = 17112,
  [28457] = 17543,
  [28460] = 17109,
  [28461] = 17540,
  [28464] = 17106,
  [28465] = 17537,
  [28468] = 17103,
  [28469] = 17534,
  [28472] = 17100,
  [28473] = 17531,
  [28476] = 17098,
  [28477] = 17528,
  [28481] = 17541,
  [28482] = 17570,
  [28483] = 17145,
  [28485] = 17538,
  [28487] = 17117,
  [28489] = 17535,
  [28492] = 17102,
  [28493] = 17532,
  [28496] = 17099,
  [28497] = 17529,
  [28500] = 17096,
  [28501] = 17526,
  [28504] = 17093,
  [28505] = 17523,
  [28508] = 17090,
  [28509] = 17520,
  [28510] = 17089,
  [28512] = 17088,
  [28513] = 17517,
  [28514] = 17087,
  [28516] = 17086,
  [28517] = 17514,
  [28518] = 17085,
  [32160] = 13804,
  [33358] = 12556,
  [33403] = 12273,
  [33405] = 12272,
  [33409] = 12270,
  [33411] = 12269,
  [33416] = 12266,
  [33420] = 20821,
  [33434] = 12111,
  [33436] = 12111,
  [33438] = 12111,
  [33440] = 12183,
  [33454] = 12119,
  [33456] = 12119,
  [33458] = 12118,
  [33459] = 12118,
  [33478] = 12445,
  [33480] = 12180,
  [33484] = 12440,
  [33487] = 12174,
  [33490] = 12435,
  [33493] = 12169,
  [33496] = 12430,
  [33498] = 12165,
  [33502] = 12425,
  [33505] = 12159,
  [33508] = 12420,
  [33511] = 12154,
  [33514] = 12415,
  [33516] = 12150,
  [33520] = 12410,
  [33523] = 12144,
  [33526] = 12362,
  [33528] = 12112,
  [33531] = 12110,
  [33537] = 12106,
  [33540] = 12104,
  [33552] = 12396,
  [33555] = 12128,
  [33557] = 12392,
  [33560] = 12124,
  [33563] = 12387,
  [33567] = 12118,
  [33569] = 12382,
  [33573] = 12113,
  [33575] = 12377,
  [33578] = 12109,
  [33581] = 12372,
  [33585] = 12103,
  [33587] = 12367,
  [33591] = 12098,
  [33593] = 12362,
  [33594] = 12096,
  [33596] = 20647,
  [33600] = 12380,
  [33602] = 12447,
  [33603] = 12018,
  [33606] = 12375,
  [33612] = 12370,
  [33614] = 11937,
  [33618] = 12417,
  [33620] = 11932,
  [33624] = 12360,
  [33626] = 11927,
  [33630] = 12355,
  [33632] = 11922,
  [33636] = 12350,
  [33638] = 11917,
  [33642] = 12345,
  [33644] = 11912,
  [33646] = 11911,
  [33648] = 12358,
  [33650] = 11974,
  [33652] = 12399,
  [33654] = 12353,
  [33660] = 12348,
  [33663] = 11916,
  [33666] = 12343,
  [33669] = 11911,
  [33672] = 12338,
  [33675] = 11906,
  [33678] = 12333,
  [33681] = 11901,
  [33684] = 12328,
  [33687] = 11896,
  [33690] = 12323,
  [33693] = 11891,
  [33694] = 11997,
  [33696] = 12336,
  [33697] = 11930,
  [33698] = 12355,
  [33699] = 11927,
  [33702] = 12331,
  [33708] = 12326,
  [33711] = 11896,
  [33714] = 12269,
  [33717] = 11891,
  [33720] = 12316,
  [33723] = 11886,
  [33726] = 12311,
  [33729] = 11881,
  [33732] = 12306,
  [33735] = 11876,
  [33738] = 12301,
  [33739] = 12342,
  [33741] = 11871,
  [33789] = 12142,
  [33795] = 12137,
  [33796] = 20575,
  [33797] = 11873,
  [33801] = 12132,
  [33802] = 11869,
  [33804] = 11868,
  [33808] = 11865,
  [33810] = 11864,
  [33813] = 12121,
  [33819] = 12116,
  [33825] = 12111,
  [33831] = 12106,
  [33837] = 12052,
  [33839] = 11809,
  [33843] = 12049,
  [33846] = 11803,
  [33849] = 12048,
  [33852] = 11798,
  [33855] = 12043,
  [33856] = 20514,
  [33857] = 11794,
  [33861] = 12042,
  [33862] = 20507,
  [33864] = 11788,
  [33867] = 12037,
  [33868] = 11923,
  [33870] = 11783,
  [33873] = 12036,
  [33879] = 12031,
  [33885] = 12071,
  [33886] = 11806,
  [33889] = 11804,
  [33891] = 12066,
  [33892] = 11802,
  [33895] = 11800,
  [33897] = 12061,
  [33898] = 11816,
  [33900] = 11815,
  [33903] = 12056,
  [33904] = 11798,
  [33906] = 11797,
  [33909] = 12051,
  [33910] = 11794,
  [33913] = 11792,
  [33915] = 12046,
  [33916] = 11790,
  [33919] = 11788,
  [33921] = 12041,
  [33922] = 11786,
  [33924] = 11785,
  [33927] = 12036,
  [33928] = 11782,
  [33931] = 11780,
  [33933] = 12055,
  [33939] = 12050,
  [33945] = 12045,
  [33947] = 11612,
  [33951] = 12040,
  [33953] = 11607,
  [33957] = 12035,
  [33959] = 11602,
  [33963] = 12030,
  [33965] = 11597,
  [33969] = 12025,
  [33971] = 11592,
  [33975] = 12020,
  [33977] = 11587,
  [33979] = 11586,
  [33981] = 12033,
  [33982] = 11643,
  [33984] = 20258,
  [33987] = 12028,
  [33993] = 12023,
  [33996] = 11591,
  [33999] = 12018,
  [34002] = 11586,
  [34005] = 12013,
  [34008] = 11581,
  [34011] = 12008,
  [34014] = 11576,
  [34017] = 12003,
  [34020] = 11571,
  [34023] = 11998,
  [34026] = 11566,
  [34027] = 12052,
  [34029] = 12011,
  [34030] = 11599,
  [34032] = 12022,
  [34033] = 11597,
  [34035] = 12006,
  [34041] = 12001,
  [34044] = 11570,
  [34047] = 11996,
  [34050] = 11565,
  [34053] = 11991,
  [34056] = 11560,
  [34059] = 11986,
  [34062] = 11555,
  [34065] = 11981,
  [34068] = 11550,
  [34071] = 11976,
  [34072] = 12010,
  [34074] = 11545,
  [42759] = 2909,
  [42784] = 3118,
  [42786] = 2859,
  [42790] = 3117,
  [42793] = 2853,
  [42796] = 3112,
  [42799] = 2848,
  [42804] = 3087,
  [42807] = 3088,
  [42808] = 2834,
  [42811] = 3085,
  [42814] = 3087,
  [43088] = 2877,
  [43089] = 2624,
  [43092] = 2605,
  [43094] = 2872,
  [43097] = 2620,
  [43124] = 2843,
  [43128] = 2570,
  [43142] = 2826,
  [43143] = 2573,
  [43154] = 2815,
  [43155] = 2563,
  [43158] = 2541,
  [46057] = -425,
  [46058] = -425,
  [46059] = -425,
  [46060] = -525,
  [46061] = -6,
  [46062] = -431,
  [46063] = -7,
  [54377] = -8677,
  [55922] = -10221,
  [57080] = -137,
  [57081] = -137,
  [57082] = -135,
  [57083] = -137,
  [57084] = -139,
  [57085] = -137,
  [57086] = -137,
  [57087] = -137,
  [57088] = -137,
  [57089] = -137,
  [57090] = -137,
  [57091] = -137,
  [57092] = -137,
  [57093] = -137,
  [57094] = -137,
  [57095] = -137,
  [57096] = -137,
  [57098] = -137,
  [57099] = -137,
  [57100] = -137,
  [57101] = -137,
  [57102] = -137,
  [57103] = -137,
  [57104] = -137,
  [57105] = -137,
  [57106] = -137,
  [57107] = -137,
  [57108] = -137,
  [57109] = -137,
  [57110] = -137,
  [57111] = -137,
  [57112] = -137,
  [57113] = -137,
  [57114] = -137,
  [57115] = -137,
  [57116] = -137,
  [57117] = -137,
  [57118] = -137,
  [57119] = -137,
  [57120] = -137,
  [57121] = -137,
  [57122] = -137,
  [57123] = -137,
  [57124] = -137,
  [57125] = -137,
  [57126] = -137,
  [57127] = -137,
  [57128] = -137,
  [57129] = -137,
  [57130] = -137,
  [57131] = -137,
  [57132] = -137,
  [57133] = -137,
  [57134] = -137,
  [57135] = -137,
  [57136] = -137,
  [57137] = -137,
  [57138] = -137,
  [57139] = -137,
  [57140] = -137,
  [57141] = -137,
  [57142] = -137,
  [57143] = -137,
  [57144] = -137,
  [57145] = -137,
  [57146] = -137,
  [57147] = -137,
  [57148] = -137,
  [57149] = -137,
  [57150] = -137,
  [57151] = -137,
  [57152] = -137,
  [57153] = -137,
  [57155] = -138,
  [57156] = -138,
  [57157] = -138,
  [57158] = -138,
  [57159] = -138,
  [57160] = -138,
  [57161] = -138,
  [57162] = -138,
  [57163] = -138,
  [57164] = -138,
  [57165] = -138,
  [57166] = -138,
  [57167] = -138,
  [57168] = -138,
  [57169] = -138,
  [57170] = -138,
  [57171] = -138,
  [57172] = -138,
  [57173] = -138,
  [57174] = -138,
  [57175] = -138,
  [57176] = -138,
  [57177] = -138,
  [57178] = -138,
  [57179] = -138,
  [57180] = -138,
  [57181] = -138,
  [57182] = -138,
  [57183] = -138,
  [57184] = -138,
  [57185] = -138,
  [57186] = -138,
  [57187] = -138,
  [57188] = -138,
  [57189] = -138,
  [57190] = -138,
  [57191] = -138,
  [57192] = -138,
  [57193] = -138,
  [57194] = -138,
  [57195] = -138,
  [57196] = -138,
  [57197] = -138,
  [57198] = -138,
  [57199] = -138,
  [57200] = -138,
  [57201] = -138,
  [57202] = -138,
  [57203] = -138,
  [57204] = -138,
  [57205] = -138,
  [57206] = -138,
  [57207] = -138,
  [57208] = -138,
  [57209] = -138,
  [57210] = -138,
  [57211] = -138,
  [57212] = -138,
  [57213] = -138,
  [57214] = -138,
  [57215] = -138,
  [57216] = -138,
  [57217] = -138,
  [64221] = 2,
  [68233] = -44,
  [68234] = -44,
  [68235] = -44,
  [68236] = -44,
  [68237] = -44,
  [68238] = -44,
  [68239] = -44,
  [68240] = -44,
  [68241] = -44,
  [68242] = -44,
  [68243] = -44,
  [68244] = -44,
  [68245] = -44,
  [68246] = -44,
  [68247] = -44,
  [68248] = -44,
  [68249] = -44,
  [68250] = -44,
  [68251] = -44,
  [68252] = -44,
  [68253] = -44,
  [68254] = -44,
  [68255] = -44,
  [68256] = -44,
  [68257] = -44,
  [68258] = -44,
  [68259] = -44,
  [68260] = -44,
  [68261] = -44,
  [68262] = -44,
  [68263] = -44,
  [68264] = -44,
  [68265] = -44,
  [68266] = -44,
  [68267] = -44,
  [68268] = -44,
  [68269] = -44,
  [68270] = -44,
  [68271] = -44,
  [68272] = -44,
  [68273] = -44,
  [68274] = -44,
  [68275] = -44,
  [68276] = -44,
  [71056] = 4,
  [71057] = 4,
  [71058] = 4,
  [71059] = 4,
  [87685] = -3,
  [87686] = -3,
  [87687] = -3,
  [87690] = -2,
  [87691] = -2,
  [87695] = -3,
  [87696] = -3,
  [87697] = -3,
  [87699] = -1,
  [101879] = -4911,
  [112425] = -15458,
  [112426] = -15460,
  [112433] = -15468,
  [112434] = -15470,
  [112435] = -15472,
  [112438] = -15476,
  [112439] = -15478,
  [112440] = -15480,
  [114327] = 4664,
  [114328] = 4664,
  [114329] = 4664,
  [114330] = 4664,
  [114331] = 4664,
  [114332] = 4665,
  [114333] = 4663,
  [114334] = 4664,
  [114335] = 4664,
  [114336] = 4664,
  [114337] = 4664,
  [114338] = 4664,
  [114339] = 4664,
  [114340] = 4664,
  [114349] = 4612,
  [114350] = 4612,
  [114351] = 4612,
  [114352] = 4612,
  [114353] = 4612,
  [114354] = 4612,
  [114355] = 4612,
  [114356] = 4612,
  [114357] = 4612,
  [114358] = 4612,
  [114359] = 4612,
  [114360] = 4612,
  [114361] = 4612,
  [114362] = 4612,
  [114363] = 4612,
  [114364] = 4612,
  [114365] = 4612,
  [114366] = 4612,
  [114367] = 4612,
  [114368] = 4612,
  [114369] = 4612,
  [114370] = 4612,
  [114371] = 4612,
  [114372] = 4612,
  [114373] = 4612,
  [114374] = 4612,
  [114375] = 4543,
  [114376] = 4543,
  [114377] = 4543,
  [114378] = 4543,
  [114379] = 4543,
  [114380] = 4543,
  [114381] = 4543,
  [114382] = 4543,
  [114383] = 4543,
  [114384] = 4543,
  [114385] = 4543,
  [114386] = 4543,
  [114387] = 4543,
  [114388] = 4543,
  [114389] = 4543,
  [114390] = 4543,
  [114391] = 4543,
  [114392] = 4543,
  [114393] = 4543,
  [114394] = 4543,
  [114395] = 4543,
  [114396] = 4543,
  [114397] = 4543,
  [114398] = 4543,
  [114399] = 4543,
  [114400] = 4543,
  [114401] = 4543,
  [114402] = 4543,
  [114403] = 4543,
  [114404] = 4543,
  [114405] = 4543,
  [114406] = 4543,
  [114407] = 4543,
  [114408] = 4543,
  [114409] = 4543,
  [114410] = 6900,
  [114411] = 4542,
  [114412] = 4542,
  [114413] = 6896,
  [114415] = 4540,
  [114416] = 4540,
  [114417] = 4540,
  [114422] = 6886,
  [114424] = 6941,
  [114426] = 4490,
  [115027] = 2,
  [115151] = 755,
  [115152] = 755,
  [115153] = 755,
  [115154] = 755,
  [115155] = 755,
  [115156] = 755,
  [115157] = 755,
  [115158] = 755,
  [115159] = 755,
  [115160] = 755,
  [115161] = 755,
  [115162] = 755,
  [115163] = 755,
  [115164] = 755,
  [115165] = 755,
  [115166] = 755,
  [115167] = 755,
  [115168] = 755,
  [115169] = 755,
  [115170] = 755,
  [115171] = 755,
  [115172] = 755,
  [115173] = 755,
  [115174] = 755,
  [115175] = 790,
  [115176] = 790,
  [115177] = 755,
  [115178] = 785,
  [115179] = 785,
  [115180] = 754,
  [115181] = 754,
  [115182] = 754,
  [115183] = 754,
  [115184] = 754,
  [115185] = 754,
  [115186] = 754,
  [115187] = 754,
  [115188] = 754,
  [115189] = 754,
  [115190] = 754,
  [115191] = 754,
  [115192] = 754,
  [115193] = 754,
  [115194] = 754,
  [115195] = 754,
  [115196] = 754,
  [115197] = 754,
  [115198] = 754,
  [115199] = 754,
  [115200] = 754,
  [115201] = 754,
  [115202] = 755,
  [115203] = 753,
  [115204] = 755,
  [115205] = 801,
  [115208] = 723,
  [115209] = 724,
  [115210] = 751,
  [115211] = 751,
  [115212] = 718,
  [115213] = 786,
  [115214] = 757,
  [115215] = 757,
  [115216] = 757,
  [115217] = 757,
  [115218] = 757,
  [115219] = 758,
  [115220] = 759,
  [115221] = 759,
  [115222] = 759,
  [115223] = 759,
  [115224] = 759,
  [115225] = 759,
  [115226] = 759,
  [115227] = 759,
  [115228] = 759,
  [115229] = 759,
  [115230] = 759,
  [115231] = 766,
  [115232] = 766,
  [115233] = 769,
  [115234] = 769,
  [115235] = 755,
  [115236] = 755,
  [115237] = 755,
  [115238] = 758,
  [115239] = 762,
  [115240] = 768,
  [115241] = 768,
  [115242] = 768,
  [115243] = 768,
  [115244] = 732,
  [115245] = 750,
  [115246] = 758,
  [115247] = 758,
  [115248] = 759,
  [115249] = 718,
  [115250] = 720,
  [115251] = 727,
  [115252] = 742,
  [115253] = 747,
  [115254] = 714,
  [115255] = 714,
  [115256] = 737,
  [115262] = 6049,
  [115272] = 740,
  [115273] = 740,
  [115274] = 740,
  [115275] = 740,
  [115276] = 740,
  [115277] = 740,
  [115278] = 740,
  [115279] = 740,
  [115280] = 740,
  [115281] = 740,
  [115282] = 740,
  [115283] = 740,
  [115284] = 740,
  [115285] = 740,
  [115286] = 740,
  [115287] = 740,
  [115288] = 740,
  [115289] = 740,
  [115290] = 740,
  [115291] = 740,
  [115292] = 740,
  [115293] = 740,
  [115294] = 740,
  [115295] = 740,
  [115296] = 740,
  [115297] = 740,
  [115298] = 740,
  [115299] = 740,
  [115300] = 740,
  [115301] = 740,
  [115302] = 740,
  [115303] = 740,
  [115304] = 740,
  [115305] = 740,
  [115306] = 740,
  [115307] = 740,
  [115308] = 740,
  [115309] = 740,
  [115310] = 740,
  [115311] = 740,
  [115312] = 740,
  [115313] = 740,
  [115314] = 740,
  [115315] = 740,
  [115316] = 740,
  [115317] = 740,
  [115318] = 740,
  [115319] = 740,
  [115320] = 740,
  [115321] = 740,
  [115322] = 740,
  [115323] = 740,
  [115324] = 740,
  [115325] = 740,
  [115326] = 740,
  [115327] = 740,
  [115328] = 740,
  [115329] = 740,
  [115330] = 740,
  [115331] = 740,
  [115332] = 740,
  [115333] = 740,
  [115334] = 740,
  [115335] = 740,
  [115337] = 740,
  [115338] = 740,
  [115339] = 740,
  [115340] = 740,
  [115351] = 796,
  [115352] = 796,
  [115353] = 796,
  [115354] = 796,
  [115355] = 796,
  [115356] = 796,
  [115357] = 796,
  [115358] = 796,
  [115359] = 796,
  [115360] = 796,
  [115361] = 796,
  [115362] = 796,
  [115363] = 796,
  [115364] = 796,
  [115365] = 796,
  [115366] = 796,
  [115367] = 796,
  [115368] = 796,
  [115369] = 796,
  [115370] = 796,
  [115371] = 796,
  [115372] = 796,
  [115373] = 796,
  [115374] = 796,
  [115375] = 796,
  [115376] = 796,
  [115377] = 796,
  [115378] = 796,
  [115379] = 796,
  [115380] = 796,
  [115381] = 796,
  [115382] = 796,
  [115383] = 796,
  [115384] = 796,
  [115385] = 796,
  [115386] = 796,
  [115387] = 796,
  [115388] = 796,
  [115389] = 796,
  [115390] = 796,
  [115391] = 796,
  [115392] = 796,
  [115393] = 796,
  [115394] = 796,
  [115396] = 795,
  [115397] = 795,
  [115398] = 795,
  [115399] = 795,
  [115400] = 795,
  [115401] = 795,
  [115402] = 795,
  [115403] = 795,
  [115404] = 795,
  [115405] = 795,
  [115406] = 795,
  [115407] = 795,
  [115408] = 795,
  [115409] = 795,
  [115410] = 795,
  [115411] = 795,
  [115413] = 794,
  [115414] = 794,
  [115415] = 794,
  [115416] = 794,
  [115417] = 794,
  [115418] = 794,
  [115419] = 794,
  [115422] = 794,
  [115429] = 292,
  [115430] = 292,
  [115431] = 292,
  [115432] = 292,
  [115433] = 292,
  [115434] = 292,
  [115435] = 292,
  [115436] = 292,
  [115437] = 292,
  [115438] = 292,
  [115439] = 292,
  [115440] = 292,
  [115441] = 292,
  [115442] = 292,
  [115443] = 292,
  [115444] = 292,
  [115445] = 292,
  [115446] = 292,
  [115447] = 292,
  [115448] = 292,
  [115449] = 292,
  [115450] = 292,
  [115451] = 292,
  [115452] = 292,
  [115453] = 292,
  [115454] = 292,
  [115455] = 292,
  [115456] = 292,
  [115457] = 292,
  [115458] = 292,
  [115459] = 292,
  [115460] = 292,
  [115461] = 292,
  [115462] = 292,
  [115463] = 292,
  [115464] = 292,
  [115465] = 292,
  [115466] = 292,
  [115467] = 292,
  [115468] = 292,
  [115469] = 292,
  [115470] = 292,
  [115471] = 292,
  [115472] = 292,
  [115473] = 292,
  [115474] = 292,
  [115475] = 292,
  [115476] = 292,
  [115477] = 292,
  [115478] = 292,
  [115479] = 292,
  [115480] = 292,
  [115481] = 292,
  [115482] = 292,
  [115483] = 292,
  [115484] = 292,
  [115485] = 292,
  [115486] = 292,
  [115487] = 292,
  [115488] = 292,
  [115489] = 292,
  [115490] = 292,
  [115491] = 292,
  [115492] = 292,
  [115493] = 292,
  [115495] = 292,
  [115496] = 292,
  [115497] = 292,
  [115498] = 292,
  [115499] = 292,
  [115500] = 292,
  [115501] = 292,
  [115502] = 292,
  [115503] = 292,
  [115504] = 292,
  [115507] = 292,
  [115508] = 292,
  [115509] = 292,
  [115510] = 292,
  [115511] = 292,
  [115512] = 292,
  [115513] = 292,
  [115514] = 292,
  [115515] = 292,
  [115516] = 292,
  [115517] = 292,
  [115518] = 292,
  [115519] = 292,
  [115520] = 292,
  [115521] = 292,
  [115522] = 292,
  [115523] = 292,
  [115524] = 292,
  [115525] = 292,
  [115526] = 292,
  [115527] = 292,
  [115528] = 292,
  [115529] = 292,
  [115530] = 292,
  [115531] = 292,
  [115532] = 292,
  [115533] = 292,
  [115534] = 292,
  [115535] = 292,
  [115536] = 292,
  [115543] = 286,
  [115544] = 286,
  [115545] = 286,
  [115546] = 286,
  [115550] = 283,
  [115551] = 283,
  [115552] = 283,
  [115553] = 283,
  [115554] = 283,
  [115555] = 283,
  [115556] = 283,
  [115557] = 283,
  [115558] = 283,
  [115559] = 283,
  [115560] = 283,
  [115561] = 283,
  [115562] = 283,
  [115563] = 283,
  [115564] = 283,
  [115565] = 283,
  [115566] = 283,
  [115567] = 283,
  [115568] = 283,
  [115569] = 283,
  [115570] = 283,
  [115571] = 283,
  [115572] = 283,
  [115573] = 283,
  [115574] = 283,
  [115575] = 283,
  [115576] = 283,
  [115579] = 281,
  [115580] = 281,
  [115581] = 281,
  [115582] = 281,
  [115583] = 281,
  [115584] = 281,
  [115585] = 281,
  [115586] = 281,
  [115587] = 281,
  [115588] = 281,
  [115589] = 281,
  [115590] = 281,
  [115591] = 281,
  [115592] = 281,
  [115593] = 281,
  [115594] = 281,
  [115595] = 281,
  [115596] = 281,
  [115597] = 281,
  [115598] = 281,
  [115599] = 281,
  [115600] = 281,
  [115601] = 281,
  [115602] = 281,
  [115603] = 281,
  [115604] = 281,
  [115605] = 281,
  [115606] = 281,
  [115607] = 281,
  [115608] = 281,
  [115609] = 281,
  [115610] = 281,
  [115611] = 281,
  [115612] = 281,
  [115613] = 281,
  [115614] = 281,
  [115615] = 281,
  [115616] = 281,
  [115617] = 281,
  [115618] = 281,
  [115619] = 281,
  [115620] = 5693,
  [115625] = 276,
  [115626] = 276,
  [115627] = 276,
  [115628] = 276,
  [115631] = 5740,
  [115637] = 444,
  [115638] = 444,
  [115639] = 444,
  [115640] = 444,
  [115641] = 444,
  [115642] = 444,
  [115643] = 444,
  [115644] = 444,
  [115645] = 444,
  [115646] = 444,
  [115647] = 444,
  [115648] = 444,
  [115649] = 444,
  [115650] = 444,
  [115651] = 444,
  [115652] = 444,
  [115653] = 444,
  [115654] = 444,
  [115655] = 444,
  [115656] = 444,
  [115657] = 444,
  [115658] = 444,
  [115659] = 444,
  [115660] = 444,
  [115661] = 444,
  [115662] = 444,
  [115663] = 444,
  [115664] = 444,
  [115665] = 444,
  [115666] = 444,
  [115667] = 444,
  [115668] = 444,
  [115669] = 444,
  [115670] = 444,
  [115671] = 444,
  [115672] = 444,
  [115673] = 444,
  [115674] = 444,
  [115675] = 444,
  [115676] = 444,
  [115677] = 444,
  [115678] = 444,
  [115679] = 5524,
  [115680] = 443,
  [115681] = 443,
  [115682] = 443,
  [115683] = 443,
  [115684] = 443,
  [115685] = 443,
  [115686] = 443,
  [115687] = 443,
  [115688] = 443,
  [115689] = 443,
  [115690] = 443,
  [115691] = 443,
  [115692] = 443,
  [115693] = 443,
  [115694] = 443,
  [115695] = 443,
  [115696] = 443,
  [115697] = 443,
  [115699] = 442,
  [115700] = 442,
  [115701] = 442,
  [115702] = 442,
  [115704] = 441,
  [115711] = 435,
  [116339] = 2648,
  [116340] = 2648,
  [116341] = 2648,
  [116342] = 2648,
  [116344] = 2614,
  [116345] = 2614,
  [116346] = 2614,
  [116347] = 2570,
  [116349] = 2656,
  [116350] = 2656,
  [116351] = 2656,
  [116352] = 2656,
  [116353] = 2656,
  [116354] = 2656,
  [116355] = 2656,
  [116356] = 2656,
  [116357] = 2656,
  [116358] = 2656,
  [116359] = 2656,
  [116360] = 2656,
  [116361] = 2656,
  [116362] = 2656,
  [116363] = 2656,
  [116364] = 2656,
  [116365] = 2656,
  [116366] = 2656,
  [116367] = 2656,
  [116368] = 2656,
  [116369] = 2656,
  [116370] = 2656,
  [116371] = 2656,
  [116372] = 2656,
  [116373] = 2656,
  [116374] = 2656,
  [116375] = 2656,
  [116376] = 2656,
  [116377] = 2656,
  [116378] = 2656,
  [116379] = 2656,
  [116380] = 2656,
  [116381] = 2656,
  [116382] = 2656,
  [116383] = 2656,
  [116384] = 2656,
  [116385] = 2656,
  [116386] = 2656,
  [116387] = 2656,
  [116388] = 2656,
  [116389] = 2656,
  [116390] = 2656,
  [116391] = 2656,
  [116392] = 2656,
  [116393] = 2656,
  [116394] = 2656,
  [116395] = 2656,
  [116396] = 2656,
  [116397] = 2656,
  [116398] = 2656,
  [116399] = 2656,
  [116400] = 2656,
  [116401] = 2656,
  [116402] = 2656,
  [116403] = 2656,
  [116404] = 2656,
  [116405] = 2656,
  [116406] = 2656,
  [116407] = 2656,
  [116408] = 2656,
  [116409] = 2656,
  [116410] = 2656,
  [116411] = 2656,
  [116412] = 2656,
  [116413] = 2656,
  [116414] = 2656,
  [116415] = 2656,
  [116416] = 2656,
  [116417] = 2656,
  [116418] = 2656,
  [116419] = 2656,
  [116421] = 2655,
  [116422] = 2655,
  [116423] = 2655,
  [116424] = 2655,
  [116425] = 2655,
  [116426] = 2655,
  [116427] = 2655,
  [116428] = 2655,
  [116429] = 2655,
  [116430] = 2655,
  [116431] = 2655,
  [116432] = 2655,
  [116434] = 2654,
  [116435] = 2654,
  [116436] = 2654,
  [116437] = 2654,
  [116438] = 2654,
  [116439] = 2654,
  [116440] = 2654,
  [116441] = 2654,
  [116442] = 4765,
  [116443] = 2653,
  [116444] = 2653,
  [116446] = 2652,
  [116447] = 2652,
  [116448] = 2652,
  [116449] = 2652,
  [116450] = 2652,
  [116451] = 2652,
  [116452] = 2652,
  [116453] = 2652,
  [116454] = 2652,
  [116455] = 2652,
  [116456] = 2652,
  [116457] = 4752,
  [116458] = 2651,
  [116459] = 2651,
  [116460] = 2651,
  [116461] = 2651,
  [116462] = 2651,
  [116463] = 2651,
  [116464] = 2651,
  [116465] = 2651,
  [116466] = 2651,
  [116467] = 2651,
  [116468] = 2651,
  [116470] = 2650,
  [116471] = 2650,
  [116476] = 4738,
  [116477] = 2645,
  [116478] = 2645,
  [116499] = 4807,
  [116513] = 4853,
  [116516] = 4791,
  [117663] = 1461,
  [117664] = 1461,
  [117665] = 1461,
  [117666] = 1461,
  [117667] = 1461,
  [117668] = 1461,
  [117669] = 1461,
  [117670] = 1461,
  [117671] = 1461,
  [117672] = 1461,
  [117673] = 1461,
  [117674] = 1461,
  [117675] = 1461,
  [117676] = 1461,
  [117677] = 1461,
  [117678] = 1461,
  [117679] = 1461,
  [117680] = 1461,
  [117681] = 1461,
  [117682] = 1461,
  [117683] = 1461,
  [117684] = 1461,
  [117685] = 1461,
  [117686] = 1461,
  [117687] = 1461,
  [117688] = 1461,
  [117689] = 1461,
  [117690] = 1461,
  [117691] = 1461,
  [117692] = 1461,
  [117693] = 1461,
  [117694] = 1461,
  [117695] = 1461,
  [117696] = 1461,
  [117697] = 1461,
  [117698] = 1461,
  [117699] = 1461,
  [117700] = 1461,
  [117701] = 1461,
  [117702] = 1461,
  [117703] = 1461,
  [117704] = 1461,
  [117705] = 1461,
  [117706] = 1461,
  [117707] = 1461,
  [117708] = 1461,
  [117709] = 1461,
  [117710] = 1461,
  [117711] = 1461,
  [117712] = 1461,
  [117713] = 1461,
  [117714] = 1461,
  [117715] = 1461,
  [117716] = 1461,
  [117717] = 1461,
  [117718] = 1461,
  [117719] = 1461,
  [117720] = 1461,
  [117721] = 1461,
  [117722] = 1461,
  [117723] = 1461,
  [117724] = 1461,
  [117725] = 1461,
  [117726] = 1461,
  [117727] = 1461,
  [117728] = 1461,
  [117729] = 1461,
  [117730] = 1461,
  [117731] = 1461,
  [117732] = 1461,
  [117733] = 1461,
  [117734] = 1461,
  [117735] = 1461,
  [117736] = 1461,
  [117737] = 1461,
  [117738] = 1461,
  [117739] = 1461,
  [117740] = 1461,
  [117741] = 1461,
  [117742] = 1461,
  [117743] = 1461,
  [117744] = 1461,
  [117745] = 1461,
  [117746] = 1461,
  [117747] = 1461,
  [117748] = 1461,
  [117749] = 1461,
  [117750] = 1461,
  [117751] = 1461,
  [117752] = 1461,
  [117753] = 1461,
  [117754] = 1461,
  [117755] = 1461,
  [117756] = 1461,
  [117757] = 1461,
  [117758] = 1461,
  [117759] = 1461,
  [117761] = 1460,
  [117762] = 1460,
  [117763] = 1460,
  [117764] = 1460,
  [117766] = 1459,
  [117767] = 1459,
  [117769] = 1459,
  [117770] = 1459,
  [117771] = 1459,
  [117772] = 1459,
  [117773] = 1459,
  [117774] = 1459,
  [117775] = 1459,
  [117776] = 1459,
  [117777] = 1459,
  [117779] = 1459,
  [117780] = 1459,
  [117781] = 1459,
  [117782] = 1459,
  [117783] = 1459,
  [117784] = 1459,
  [117785] = 1459,
  [117786] = 1459,
  [117787] = 1459,
  [117788] = 1459,
  [117789] = 1459,
  [117790] = 1459,
  [117791] = 1459,
  [117792] = 1459,
  [117793] = 1459,
  [117794] = 1459,
  [117795] = 1459,
  [117796] = 1459,
  [117797] = 1459,
  [117798] = 1459,
  [117799] = 1459,
  [117800] = 1459,
  [117801] = 1459,
  [117802] = 1459,
  [117803] = 1459,
  [117804] = 1459,
  [117805] = 1459,
  [117806] = 1459,
  [117807] = 1459,
  [117808] = 1459,
  [117809] = 1459,
  [117810] = 1459,
  [117811] = 1459,
  [117812] = 1459,
  [117813] = 1459,
  [117814] = 1459,
  [117815] = 1459,
  [117816] = 1459,
  [117817] = 1459,
  [117818] = 1459,
  [117819] = 1459,
  [117820] = 1459,
  [117821] = 1459,
  [117822] = 1459,
  [117823] = 1459,
  [117824] = 1459,
  [117825] = 1459,
  [117826] = 1459,
  [117827] = 1459,
  [117828] = 1459,
  [117829] = 1459,
  [117839] = 1451,
  [117840] = 1451,
  [117841] = 1451,
  [117842] = 1451,
  [117844] = 1451,
  [117845] = 1451,
  [117846] = 1451,
  [117847] = 1451,
  [117848] = 1451,
  [117849] = 1451,
  [117850] = 1451,
  [117851] = 1451,
  [117852] = 1451,
  [117853] = 1451,
  [117854] = 1451,
  [117855] = 1451,
  [117856] = 1451,
  [117857] = 1451,
  [117858] = 1451,
  [117859] = 1451,
  [117860] = 1451,
  [117861] = 1451,
  [117883] = 3486,
  [117900] = 3645,
  [117910] = 3395,
  [117911] = 3393,
  [117926] = 1388,
  [117929] = 1388,
  [117937] = 1388,
  [117940] = 1388,
  [117942] = 1388,
  [117943] = 1388,
  [117950] = 1388,
  [117954] = 1388,
  [117956] = 1388,
  [117960] = 1388,
  [117963] = 1387,
  [117991] = 3100,
  [117996] = 1355,
  [117997] = 1355,
  [117998] = 1355,
  [117999] = 1355,
  [118001] = 1355,
  [118002] = 1355,
  [118003] = 1355,
  [118004] = 1355,
  [118005] = 1355,
  [118006] = 1355,
  [118007] = 1355,
  [118008] = 1355,
  [118009] = 1355,
  [118010] = 1355,
  [118011] = 1355,
  [118012] = 1355,
  [118013] = 1355,
  [118014] = 1355,
  [118015] = 1355,
  [118016] = 1355,
  [118017] = 1355,
  [118018] = 1355,
  [118019] = 1355,
  [118020] = 1355,
  [118021] = 1355,
  [118022] = 1355,
  [118023] = 1355,
  [118024] = 1355,
  [118025] = 1355,
  [118026] = 1355,
  [118027] = 1355,
  [118028] = 1355,
  [118029] = 1355,
  [118030] = 1355,
  [118031] = 1355,
  [118032] = 1355,
  [118033] = 1355,
  [118034] = 1355,
  [118035] = 1355,
  [118036] = 1355,
  [118037] = 1355,
  [118038] = 1355,
  [118039] = 1355,
  [118040] = 1355,
  [118041] = 1355,
  [118042] = 1355,
  [118044] = 1355,
  [118045] = 1355,
  [118046] = 1355,
  [118047] = 1355,
  [118048] = 1355,
  [118049] = 1355,
  [118050] = 1355,
  [118051] = 1355,
  [118052] = 1355,
  [118053] = 3045,
  [118063] = 3041,
  [118067] = 1341,
  [118080] = 3040,
  [118083] = 1330,
  [118086] = 1330,
  [118088] = 1330,
  [118090] = 1330,
  [118091] = 1330,
  [118092] = 1330,
  [118093] = 1330,
  [118094] = 1330,
  [118095] = 1330,
  [118097] = 1330,
  [118099] = 1330,
  [118102] = 1330,
  [118108] = 1330,
  [118109] = 1330,
  [118110] = 1330,
  [118113] = 1330,
  [118114] = 1330,
  [118115] = 1330,
  [118116] = 1330,
  [118122] = 1329,
  [118133] = 1329,
  [118134] = 1329,
  [118135] = 1329,
  [118136] = 1329,
  [118176] = 1291,
  [118177] = 1291,
  [118178] = 1291,
  [118179] = 1291,
  [118180] = 1291,
  [118181] = 1291,
  [118183] = 3189,
  [118185] = 1291,
  [118186] = 1291,
  [118187] = 1291,
  [118188] = 1291,
  [118189] = 1291,
  [118190] = 1291,
  [118191] = 1291,
  [118192] = 1291,
  [118193] = 1291,
  [118194] = 1291,
  [118195] = 1291,
  [118196] = 1291,
  [118197] = 1291,
  [118198] = 1291,
  [118199] = 1291,
  [118200] = 1291,
  [118233] = 2930,
  [118234] = 2930,
  [118235] = 2930,
  [118236] = 2930,
  [118238] = 2930,
  [118303] = 2894,
  [118305] = 2895,
  [118306] = 2893,
  [118334] = 1184,
  [118336] = 1184,
  [118340] = 1184,
  [118341] = 1184,
  [118342] = 1184,
  [118343] = 1184,
  [118345] = 1184,
  [118348] = 1184,
  [118349] = 1184,
  [118355] = 1184,
  [118356] = 1184,
  [118357] = 1184,
  [118358] = 1184,
  [118359] = 1184,
  [118553] = 1039,
  [120076] = 1,
  [120410] = 649,
  [120436] = 331,
  [120762] = 6,
  [120763] = 6,
  [120764] = 6,
  [121050] = 324,
  [121051] = 322,
  [121314] = 1,
  [121538] = 8,
  [125486] = 1350,
  [125510] = 1351,
  [125523] = 1332,
  [125524] = 1330,
  [125525] = 1328,
  [125526] = 1326,
  [125527] = 1324,
  [125528] = 1321,
  [125530] = 1333,
  [125531] = 1325,
  [125533] = 1325,
  [125535] = 1322,
  [126035] = 547,
  [126036] = 547,
  [126120] = 718,
  [126123] = 716,
  [126124] = 713,
  [126125] = 710,
  [126126] = 708,
  [126133] = 726,
  [126134] = 762,
  [126135] = 762,
  [126147] = 954,
  [126151] = 951,
  [126338] = 573,
  [126339] = 577,
  [126340] = 586,
  [126341] = 586,
  [126342] = 595,
  [126343] = 611,
  [126344] = 623,
  [126345] = 628,
  [126346] = 628,
  [126347] = 645,
  [126348] = 645,
  [126349] = 652,
  [126350] = 660,
  [126351] = 660,
  [126352] = 661,
  [126353] = 672,
  [126354] = 679,
  [126355] = 679,
  [126356] = 679,
  [126357] = 706,
  [126358] = 736,
  [126359] = 736,
  [126360] = 743,
  [126361] = 669,
  [126362] = 669,
  [126363] = 669,
  [126364] = 692,
  [126365] = 693,
  [126366] = 693,
  [126367] = 707,
  [126368] = 718,
  [126369] = 721,
  [126370] = 713,
  [126371] = 716,
  [126372] = 590,
  [126373] = 590,
  [126374] = 690,
  [126375] = 691,
  [126376] = 691,
  [126377] = 640,
  [126378] = 640,
  [126379] = 640,
  [126380] = 611,
  [126381] = 619,
  [126382] = 639,
  [126383] = 611,
  [126384] = 611,
  [126385] = 613,
  [126386] = 613,
  [126387] = 609,
  [126388] = 609,
  [126389] = 615,
  [126390] = 615,
  [126391] = 611,
  [126392] = 611,
  [126394] = 674,
  [126395] = 674,
  [126396] = 674,
  [126397] = 523,
  [126398] = 523,
  [126399] = 524,
  [126400] = 524,
  [126401] = 514,
  [126402] = 515,
  [126403] = 565,
  [126404] = 562,
  [126405] = 570,
  [126406] = 570,
  [126407] = 528,
  [126408] = 526,
  [126409] = 524,
  [126410] = 497,
  [126411] = 498,
  [126412] = 502,
  [126413] = 516,
  [126414] = 517,
  [126415] = 525,
  [126416] = 534,
  [126417] = 534,
  [126418] = 560,
  [126419] = 560,
  [126420] = 560,
  [126421] = 560,
  [126422] = 560,
  [126423] = 560,
  [126424] = 560,
  [126425] = 560,
  [126426] = 560,
  [126427] = 560,
  [126428] = 560,
  [126429] = 560,
  [126430] = 560,
  [126431] = 491,
  [126432] = 493,
  [126433] = 642,
  [126434] = 643,
  [126435] = 484,
  [126436] = 482,
  [126437] = 532,
  [126438] = 532,
  [126439] = 521,
  [126440] = 519,
  [126441] = 520,
  [126442] = 514,
  [126443] = 463,
  [126444] = 464,
  [126445] = 465,
  [126446] = 466,
  [126447] = 481,
  [126448] = 482,
  [126449] = 487,
  [126450] = 488,
  [126451] = 462,
  [126452] = 487,
  [126453] = 518,
  [126454] = 518,
  [126455] = 491,
  [126456] = 499,
  [126457] = 595,
  [126458] = 597,
  [126459] = 595,
  [126460] = 593,
  [126461] = 590,
  [126471] = 627,
  [126472] = 628,
  [126473] = 623,
  [126474] = 625,
  [126480] = 617,
  [126482] = 609,
  [126483] = 609,
  [126484] = 609,
  [126485] = 603,
  [126486] = 603,
  [126487] = 589,
  [126488] = 597,
  [126489] = 595,
  [126490] = 591,
  [126491] = 591,
  [126492] = 588,
  [126493] = 586,
  [126494] = 584,
  [126495] = 453,
  [126496] = 481,
  [126497] = 446,
  [126498] = 459,
  [126499] = 433,
  [126500] = 442,
  [126501] = 440,
  [126502] = 443,
  [126503] = 444,
  [126504] = 445,
  [126505] = 439,
  [126506] = 452,
  [126507] = 445,
  [126508] = 445,
  [126509] = 393,
  [126510] = 393,
  [126511] = 393,
  [126512] = 508,
  [126513] = 387,
  [126514] = 391,
  [126515] = 386,
  [126516] = 449,
  [126517] = 447,
  [126518] = 547,
  [126519] = 495,
  [126520] = 495,
  [126521] = 495,
  [126522] = 540,
  [126523] = 527,
  [126524] = 500,
  [126525] = 497,
  [126526] = 497,
  [126527] = 481,
  [126528] = 481,
  [126529] = 477,
  [126530] = 477,
  [126531] = 481,
  [126532] = 517,
  [126533] = 513,
  [126534] = 513,
  [126535] = 513,
  [126536] = 493,
  [126537] = 504,
  [126538] = 507,
  [126539] = 489,
  [126540] = 500,
  [126541] = 503,
  [126542] = 485,
  [126543] = 496,
  [126544] = 499,
  [126545] = 481,
  [126546] = 492,
  [126547] = 495,
  [126548] = 488,
  [126549] = 488,
  [126550] = 507,
  [126551] = 509,
  [126552] = 509,
  [126553] = 518,
  [126554] = 518,
  [126555] = 518,
  [126556] = 289,
  [126557] = 290,
  [126558] = 288,
  [126655] = 230,
  [126656] = 232,
  [126657] = 232,
  [126658] = 204,
  [126661] = 208,
  [126662] = 205,
  [126663] = 227,
  [126664] = 223,
  [126665] = 215,
  [126666] = 215,
  [126667] = 217,
  [126668] = 211,
  [126669] = 209,
  [126670] = 203,
  [126671] = 201,
  [126672] = 210,
  [126673] = 202,
  [126674] = 200,
  [126675] = 202,
  [126676] = 200,
  [126677] = 189,
  [126678] = 193,
  [126679] = 215,
  [126680] = 213,
  [126681] = 211,
  [126682] = 209,
  [126683] = 215,
  [126684] = 215,
  [126685] = 165,
  [126687] = 161,
  [126688] = 182,
  [126689] = 171,
  [126690] = 196,
  [126691] = 174,
  [126692] = 172,
  [126693] = 175,
  [126694] = 189,
  [126695] = 200,
  [126696] = 148,
  [126697] = 145,
  [126698] = 143,
  [126699] = 141,
  [126831] = 273,
  [130066] = 2118,
  [130068] = 2115,
  [130076] = 2109,
  [130077] = 2109,
  [130198] = 1989,
  [130200] = 1988,
  [130203] = 2141,
  [130208] = 2135,
  [130209] = 2133,
  [130214] = 1975,
  [130320] = 1847,
  [130326] = 1847,
  [130334] = 1845,
  [130337] = 1859,
  [132159] = 31,
  [132160] = 31,
  [132161] = 31,
  [132162] = 31,
  [132163] = 31,
  [132164] = 31,
  [133554] = -3,
  [133555] = -3,
  [133556] = -3,
  [134306] = 170,
  [134307] = 170,
  [134308] = 170,
  [134309] = 170,
  [134310] = 170,
  [134311] = 170,
  [134312] = 170,
  [134313] = 170,
  [134314] = 170,
  [134315] = 170,
  [134316] = 170,
  [134317] = 170,
  [134318] = 170,
  [134319] = 170,
  [134320] = 170,
  [134321] = 170,
  [134322] = 170,
  [134324] = 169,
  [134325] = 169,
  [134327] = 169,
  [134328] = 169,
  [134329] = 169,
  [134331] = 169,
  [134332] = 169,
  [134333] = 169,
  [134334] = 169,
  [134335] = 169,
  [134336] = 169,
  [134338] = 168,
  [134339] = 168,
  [134340] = 168,
  [134341] = 168,
  [134342] = 168,
  [134343] = 168,
  [134344] = 168,
  [134345] = 168,
  [134346] = 168,
  [134347] = 168,
  [134348] = 168,
  [134349] = 168,
  [134350] = 168,
  [134351] = 168,
  [134352] = 168,
  [134353] = 168,
  [134354] = 168,
  [134355] = 168,
  [134356] = 168,
  [134357] = 168,
  [134358] = 168,
  [134359] = 168,
  [134360] = 168,
  [134361] = 168,
  [134362] = 168,
  [134363] = 168,
  [134364] = 168,
  [134365] = 168,
  [134366] = 168,
  [134367] = 168,
  [134368] = 168,
  [134369] = 168,
  [134370] = 168,
  [134371] = 168,
  [134372] = 168,
  [134375] = 168,
  [134376] = 168,
  [134377] = 168,
  [134378] = 168,
  [134564] = 18,
  [134738] = 245,
  [134915] = 84,
  [134916] = 68,
  [134917] = 70,
  [134918] = 67,
  [134919] = 63,
  [134920] = 66,
  [134959] = 39,
  [139016] = -4,
  [139018] = -1,
  [139086] = 398,
  [139087] = 398,
  [139091] = 395,
  [139164] = 323,
  [139165] = 323,
  [139166] = 323,
  [139167] = 323,
  [139168] = 323,
  [139169] = 323,
  [139170] = 323,
  [139171] = 323,
  [139172] = 323,
  [139173] = 323,
  [139174] = 323,
  [139175] = 323,
  [139176] = 323,
  [139177] = 323,
  [139178] = 323,
  [139179] = 323,
  [139180] = 323,
  [139181] = 323,
  [139182] = 323,
  [139183] = 323,
  [139184] = 323,
  [139185] = 323,
  [139186] = 323,
  [139187] = 323,
  [139188] = 323,
  [139189] = 323,
  [139190] = 323,
  [139191] = 323,
  [139192] = 323,
  [139193] = 323,
  [139194] = 323,
  [139195] = 323,
  [139196] = 323,
  [139197] = 323,
  [139198] = 323,
  [139199] = 323,
  [139200] = 323,
  [139201] = 323,
  [139202] = 323,
  [139203] = 323,
  [139204] = 323,
  [139205] = 323,
  [139206] = 323,
  [139207] = 323,
  [139208] = 323,
  [139209] = 323,
  [139210] = 323,
  [139211] = 323,
  [139212] = 323,
  [139213] = 323,
  [139214] = 323,
  [139215] = 323,
  [139216] = 323,
  [139217] = 323,
  [139218] = 323,
  [139219] = 323,
  [139220] = 323,
  [139221] = 323,
  [139222] = 323,
  [139223] = 323,
  [139224] = 323,
  [139225] = 323,
  [139226] = 323,
  [139227] = 323,
  [139228] = 323,
  [139229] = 323,
  [139230] = 323,
  [139231] = 323,
  [139232] = 323,
  [139233] = 323,
  [139234] = 323,
  [139235] = 323,
  [139236] = 323,
  [139240] = 323,
  [139241] = 323,
  [139242] = 323,
  [139243] = 323,
  [139244] = 323,
  [139245] = 323,
  [139246] = 323,
  [139247] = 323,
  [139248] = 323,
  [139249] = 323,
  [139250] = 323,
  [139251] = 323,
  [139252] = 323,
  [139253] = 323,
  [139254] = 323,
  [139255] = 323,
  [139256] = 323,
  [139257] = 323,
  [139258] = 323,
  [139259] = 323,
  [139260] = 323,
  [139261] = 323,
  [139262] = 323,
  [139263] = 323,
  [139264] = 323,
  [139265] = 323,
  [139266] = 323,
  [139267] = 323,
  [139268] = 323,
  [139269] = 323,
  [139270] = 323,
  [139271] = 323,
  [139272] = 323,
  [139273] = 323,
  [139274] = 323,
  [139275] = 323,
  [139276] = 323,
  [139277] = 323,
  [139278] = 323,
  [139279] = 323,
  [139280] = 323,
  [139281] = 323,
  [139282] = 323,
  [139283] = 323,
  [139284] = 323,
  [139285] = 323,
  [139286] = 323,
  [139287] = 323,
  [139288] = 323,
  [139289] = 323,
  [139290] = 323,
  [139291] = 323,
  [139292] = 323,
  [139293] = 323,
  [139294] = 323,
  [139295] = 323,
  [139296] = 323,
  [139297] = 323,
  [139298] = 323,
  [139299] = 323,
  [139300] = 323,
  [139301] = 323,
  [139303] = 322,
  [139304] = 322,
  [139305] = 322,
  [139306] = 322,
  [139307] = 322,
  [139308] = 322,
  [139309] = 322,
  [139310] = 322,
  [139311] = 322,
  [139312] = 322,
  [139313] = 322,
  [139314] = 322,
  [139315] = 322,
  [139316] = 322,
  [139317] = 322,
  [139318] = 322,
  [139319] = 322,
  [139320] = 322,
  [139321] = 322,
  [139322] = 322,
  [139323] = 322,
  [139324] = 322,
  [139325] = 322,
  [139375] = 273,
  [139390] = 259,
  [141822] = 79,
  [141825] = 77,
  [141826] = 77,
  [141827] = 77,
  [141828] = 77,
  [141829] = 77,
  [141830] = 77,
  [141831] = 65,
  [141837] = 60,
  [141838] = 60,
  [141839] = 60,
  [141852] = 48,
  [145323] = 592,
  [145324] = 620,
  [145325] = 620,
  [145326] = 620,
  [145327] = 620,
  [145328] = 620,
  [145329] = 620,
  [145330] = 620,
  [145331] = 620,
  [145332] = 620,
  [145333] = 620,
  [145334] = 620,
  [145335] = 620,
  [145336] = 620,
  [145337] = 620,
  [145338] = 620,
  [145339] = 620,
  [145340] = 620,
  [145341] = 620,
  [145342] = 620,
  [145343] = 620,
  [145344] = 620,
  [145345] = 620,
  [145346] = 620,
  [145347] = 620,
  [145348] = 620,
  [145349] = 620,
  [145350] = 620,
  [145351] = 620,
  [145352] = 620,
  [145353] = 620,
  [145354] = 620,
  [145355] = 620,
  [145356] = 620,
  [145357] = 620,
  [145358] = 620,
  [145359] = 620,
  [145360] = 620,
  [145361] = 620,
  [145362] = 620,
  [145363] = 620,
  [145364] = 620,
  [145365] = 620,
  [145366] = 620,
  [145367] = 620,
  [145368] = 620,
  [145369] = 620,
  [145370] = 620,
  [145371] = 620,
  [145372] = 620,
  [145373] = 620,
  [145374] = 620,
  [145375] = 620,
  [145376] = 620,
  [145377] = 620,
  [145378] = 620,
  [145379] = 620,
  [145380] = 620,
  [145381] = 620,
  [145382] = 620,
  [145383] = 620,
  [145384] = 620,
  [145385] = 620,
  [145386] = 620,
  [145387] = 620,
  [145388] = 620,
  [145389] = 620,
  [145390] = 620,
  [145391] = 620,
  [145392] = 620,
  [145393] = 620,
  [145394] = 620,
  [145395] = 620,
  [145409] = 607,
  [145548] = 469,
  [147592] = 59,
  [147593] = 59,
  [147594] = 59,
  [147595] = 59,
  [147596] = 59,
  [147597] = 59,
  [147598] = 59,
  [151626] = 346,
  [151627] = 346,
  [151628] = 346,
  [151629] = 346,
  [151630] = 346,
  [151631] = 346,
  [151632] = 346,
  [151633] = 346,
  [151634] = 346,
  [151635] = 346,
  [151636] = 346,
  [151637] = 346,
  [151638] = 346,
  [151639] = 346,
  [151640] = 346,
  [151641] = 346,
  [151642] = 346,
  [151644] = 345,
  [151645] = 345,
  [151646] = 345,
  [151647] = 345,
  [151648] = 345,
  [151649] = 345,
  [151650] = 345,
  [151651] = 345,
  [151652] = 345,
  [151653] = 345,
  [151654] = 345,
  [151655] = 345,
  [151656] = 345,
  [151657] = 345,
  [151658] = 345,
  [151659] = 345,
  [151660] = 345,
  [151661] = 345,
  [151662] = 345,
  [151663] = 345,
  [151664] = 345,
  [151665] = 345,
  [151666] = 345,
  [151667] = 345,
  [151668] = 345,
  [151669] = 345,
  [151670] = 345,
  [151671] = 345,
  [151672] = 345,
  [151673] = 345,
  [151674] = 345,
  [151675] = 345,
  [151676] = 345,
  [151677] = 345,
  [151678] = 345,
  [151679] = 345,
  [151680] = 345,
  [151681] = 345,
  [151682] = 345,
  [151683] = 345,
  [151684] = 345,
  [151685] = 345,
  [151686] = 345,
  [151687] = 345,
  [151689] = 344,
  [151690] = 344,
  [151691] = 344,
  [151692] = 344,
  [151693] = 344,
  [151694] = 344,
  [151695] = 344,
  [151696] = 344,
  [151697] = 344,
  [151698] = 344,
  [151699] = 344,
  [151700] = 344,
  [151701] = 344,
  [151702] = 344,
  [151703] = 344,
  [151704] = 344,
  [151705] = 344,
  [151706] = 344,
  [151707] = 344,
  [151708] = 344,
  [151709] = 344,
  [151710] = 344,
  [151711] = 344,
  [151712] = 344,
  [151713] = 344,
  [151714] = 344,
  [151715] = 344,
  [151716] = 344,
  [151717] = 344,
  [151718] = 344,
  [151719] = 344,
  [151720] = 344,
  [151721] = 344,
  [151722] = 344,
  [151723] = 344,
  [151724] = 344,
  [151725] = 344,
  [151726] = 344,
  [151727] = 344,
  [151728] = 344,
  [151729] = 344,
  [151730] = 344,
  [151731] = 344,
  [151732] = 344,
  [151733] = 344,
  [151734] = 344,
  [151735] = 344,
  [151736] = 344,
  [151737] = 344,
  [151738] = 344,
  [151739] = 344,
  [151740] = 344,
  [151741] = 344,
  [151742] = 344,
  [151743] = 344,
  [151744] = 344,
  [151745] = 344,
  [151746] = 344,
  [151747] = 344,
  [151748] = 344,
  [151749] = 344,
  [151750] = 344,
  [151751] = 344,
  [151752] = 344,
  [151753] = 344,
  [151754] = 344,
  [151755] = 344,
  [151756] = 344,
  [151757] = 344,
  [151758] = 344,
  [151759] = 344,
  [151760] = 344,
  [151761] = 344,
  [151762] = 344,
  [151763] = 344,
  [151764] = 344,
  [151765] = 344,
  [151766] = 344,
  [151767] = 344,
  [151768] = 344,
  [151769] = 344,
  [151770] = 344,
  [151771] = 344,
  [151772] = 344,
  [151773] = 344,
  [151774] = 344,
  [151775] = 344,
  [151839] = 281,
  [151902] = 335,
  [151903] = 335,
  [153625] = -1,
  [153627] = -1,
  [153629] = -1,
  [153651] = 78,
  [153652] = 78,
  [153653] = 78,
  [153654] = 78,
  [153655] = 78,
  [153656] = 78,
  [153657] = 78,
  [153670] = 53,
  [153671] = 55,
  [153672] = 55,
  [153673] = 51,
  [153674] = 54,
  [153703] = 22,
  [159440] = 58,
  [159441] = 58,
  [159442] = 58,
  [159443] = 58,
  [159444] = 58,
  [159445] = 58,
  [159446] = 58,
  [165579] = 1148,
  [165580] = 1148,
  [165581] = 1148,
  [165582] = 1148,
  [165583] = 1148,
  [165584] = 1148,
  [165585] = 1148,
  [165586] = 1148,
  [165587] = 1148,
  [165588] = 1148,
  [165589] = 1148,
  [165590] = 1148,
  [165591] = 1148,
  [165592] = 1148,
  [165593] = 1148,
  [165594] = 1148,
  [165595] = 1148,
  [165596] = 1148,
  [165598] = 1147,
  [165599] = 1147,
  [165600] = 1147,
  [165601] = 1147,
  [165602] = 1147,
  [165603] = 1147,
  [165604] = 1147,
  [165605] = 1147,
  [165606] = 1147,
  [165607] = 1147,
  [165608] = 1147,
  [165609] = 1147,
  [165610] = 1147,
  [165611] = 1147,
  [165612] = 1147,
  [165613] = 1147,
  [165614] = 1147,
  [165615] = 1147,
  [165616] = 1147,
  [165617] = 1147,
  [165618] = 1147,
  [165619] = 1147,
  [165620] = 1147,
  [165621] = 1147,
  [165622] = 1147,
  [165623] = 1147,
  [165624] = 1147,
  [165625] = 1147,
  [165626] = 1147,
  [165627] = 1147,
  [165628] = 1147,
  [165629] = 1147,
  [165630] = 1147,
  [165631] = 1147,
  [165632] = 1147,
  [165633] = 1147,
  [165634] = 1147,
  [165635] = 1147,
  [165636] = 1147,
  [165637] = 1147,
  [165638] = 1147,
  [165639] = 1147,
  [165640] = 1147,
  [165641] = 1147,
  [165642] = 1147,
  [165643] = 1147,
  [165644] = 1147,
  [165645] = 1147,
  [165646] = 1147,
  [165647] = 1147,
  [165648] = 1147,
  [165649] = 1147,
  [165650] = 1147,
  [165651] = 1147,
  [165652] = 1147,
  [165653] = 1147,
  [165654] = 1147,
  [165655] = 1147,
  [165656] = 1147,
  [165657] = 1147,
  [165658] = 1147,
  [165659] = 1147,
  [165660] = 1147,
  [165661] = 1147,
  [165662] = 1147,
  [165663] = 1147,
  [165664] = 1147,
  [165665] = 1147,
  [165666] = 1147,
  [165667] = 1147,
  [165668] = 1147,
  [165669] = 1147,
  [165670] = 1147,
  [165671] = 1147,
  [165672] = 1147,
  [165673] = 1147,
  [165674] = 1147,
  [165675] = 1147,
  [165676] = 1147,
  [165677] = 1147,
  [165678] = 1147,
  [165679] = 1147,
  [165680] = 1147,
  [165681] = 1147,
  [165682] = 1147,
  [165683] = 1147,
  [165684] = 1147,
  [165685] = 1147,
  [165686] = 1147,
  [165687] = 1147,
  [165688] = 1147,
  [165689] = 1147,
  [165690] = 1147,
  [165691] = 1147,
  [165692] = 1147,
  [165693] = 1147,
  [165694] = 1147,
  [165695] = 1147,
  [165696] = 1147,
  [165697] = 1147,
  [165698] = 1147,
  [165699] = 1147,
  [165700] = 1147,
  [165701] = 1147,
  [165702] = 1147,
  [165703] = 1147,
  [165704] = 1147,
  [165705] = 1147,
  [165706] = 1147,
  [165707] = 1147,
  [165708] = 1147,
  [165709] = 1147,
  [165710] = 1147,
  [165711] = 1147,
  [165712] = 1147,
  [165713] = 1147,
  [165714] = 1147,
  [165715] = 1147,
  [165716] = 1147,
  [165717] = 1147,
  [165718] = 1147,
  [165719] = 1147,
  [165720] = 1147,
  [165721] = 1147,
  [165722] = 1147,
  [165723] = 1147,
  [165724] = 1147,
  [165725] = 1147,
  [165726] = 1147,
  [165727] = 1147,
  [165728] = 1147,
  [165729] = 1147,
  [165730] = 1147,
  [165731] = 1147,
  [165732] = 1147,
  [165733] = 1147,
  [165734] = 1147,
  [165735] = 1147,
  [165736] = 1147,
  [165737] = 1147,
  [165738] = 1147,
  [165739] = 1147,
  [165740] = 1147,
  [165741] = 1147,
  [165742] = 1147,
  [165743] = 1147,
  [165744] = 1147,
  [165745] = 1147,
  [165746] = 1147,
  [165747] = 1147,
  [165748] = 1147,
  [165749] = 1147,
  [165750] = 1147,
  [165751] = 1147,
  [165752] = 1147,
  [165753] = 1147,
  [165754] = 1147,
  [165755] = 1147,
  [165756] = 1147,
  [165757] = 1147,
  [165758] = 1147,
  [165759] = 1147,
  [165760] = 1147,
  [165761] = 1147,
  [165762] = 1147,
  [165763] = 1147,
  [165764] = 1147,
  [165765] = 1147,
  [165766] = 1147,
  [165767] = 1147,
  [165768] = 1147,
  [165769] = 1147,
  [165770] = 1147,
  [165771] = 1147,
  [165772] = 1147,
  [165773] = 1147,
  [165774] = 1147,
  [165775] = 1147,
  [165776] = 1147,
  [165777] = 1147,
  [165778] = 1147,
  [165779] = 1147,
  [165780] = 1147,
  [165781] = 1147,
  [165782] = 1147,
  [165783] = 1147,
  [165784] = 1147,
  [165785] = 1147,
  [165786] = 1147,
  [165787] = 1147,
  [165788] = 1147,
  [165789] = 1147,
  [165790] = 1147,
  [165791] = 1147,
  [165792] = 1147,
  [165793] = 1147,
  [165794] = 1147,
  [165795] = 1147,
  [165796] = 1147,
  [165797] = 1147,
  [165798] = 1147,
  [165799] = 1147,
  [165800] = 1147,
  [165801] = 1147,
  [165802] = 1147,
  [165803] = 1147,
  [165804] = 1147,
  [165805] = 1147,
  [165806] = 1147,
  [165807] = 1147,
  [165808] = 1147,
  [165846] = 1110,
  [165847] = 1110,
  [166009] = 949,
  [166450] = 509,
  [167304] = 57,
  [167311] = 43,
  [167312] = 43,
  [167313] = 43,
  [167314] = 48,
  [167315] = 48,
  [167316] = 48,
  [167317] = 54,
  [167318] = 47,
  [167319] = 38,
  [167320] = 46,
  [167321] = 51,
  [167322] = 51,
  [167323] = 51,
  [167324] = 34,
  [167325] = 50,
  [167326] = 50,
  [167327] = 40,
  [167328] = 40,
  [167329] = 40,
  [167330] = 29,
  [167331] = 29,
  [167333] = 37,
  [167335] = 42,
  [167336] = 42,
  [167337] = 42,
  [167338] = 42,
  [167339] = 42,
  [167340] = 42,
  [167341] = 42,
  [167342] = 42,
  [171323] = 1,
  [171329] = 2,
  [171333] = 148,
  [171334] = 148,
  [171335] = 148,
  [171336] = 148,
  [171337] = 148,
  [171338] = 148,
  [171339] = 148,
  [171340] = 148,
  [171341] = 148,
  [171342] = 148,
  [171343] = 148,
  [171344] = 148,
  [171345] = 148,
  [171346] = 148,
  [171347] = 148,
  [171348] = 148,
  [171349] = 148,
  [171350] = 148,
  [171351] = 148,
  [171352] = 148,
  [171353] = 148,
  [171354] = 148,
  [171355] = 148,
  [171356] = 148,
  [171357] = 148,
  [171358] = 148,
  [171359] = 148,
  [171360] = 148,
  [171361] = 148,
  [171362] = 148,
  [171363] = 148,
  [171364] = 148,
  [171365] = 148,
  [171366] = 148,
  [171367] = 148,
  [171368] = 148,
  [171369] = 148,
  [171370] = 148,
  [171371] = 148,
  [171372] = 148,
  [171373] = 148,
  [171374] = 148,
  [171375] = 148,
  [171376] = 148,
  [171377] = 148,
  [171378] = 148,
  [171379] = 148,
  [171380] = 148,
  [171381] = 148,
  [171404] = 397,
  [171405] = 397,
  [171406] = 397,
  [171407] = 397,
  [171408] = 397,
  [171409] = 397,
  [171410] = 397,
  [171432] = 3,
  [171755] = 26,
  [171756] = 26,
  [171757] = 26,
  [171758] = 26,
  [171759] = 26,
  [171760] = 26,
  [171761] = 26,
  [171762] = 26,
  [171763] = 26,
  [171764] = 26,
  [171765] = 26,
  [171766] = 26,
  [171767] = 26,
  [171768] = 26,
  [171769] = 26,
  [171770] = 26,
  [171771] = 26,
  [171772] = 26,
  [171773] = 26,
  [171774] = 26,
  [175583] = 332,
  [175584] = 332,
  [175585] = 332,
  [175586] = 332,
  [175587] = 332,
  [175588] = 332,
  [175589] = 332,
  [175590] = 332,
  [175591] = 332,
  [175592] = 332,
  [175593] = 332,
  [175594] = 332,
  [175595] = 332,
  [175596] = 332,
  [175597] = 332,
  [175598] = 332,
  [175599] = 332,
  [175600] = 332,
  [175601] = 332,
  [175602] = 332,
  [175603] = 332,
  [175604] = 332,
  [175605] = 332,
  [175606] = 332,
  [175607] = 332,
  [175608] = 332,
  [175609] = 332,
  [175610] = 332,
  [175611] = 332,
  [175612] = 332,
  [175613] = 332,
  [175614] = 332,
  [175615] = 332,
  [175616] = 332,
  [175617] = 332,
  [175618] = 332,
  [175619] = 332,
  [175620] = 332,
  [175621] = 332,
  [175622] = 332,
  [175623] = 332,
  [175624] = 332,
  [175625] = 332,
  [175626] = 332,
  [175627] = 332,
  [175628] = 332,
  [175629] = 332,
  [175630] = 332,
  [175631] = 332,
  [175632] = 332,
  [175633] = 332,
  [175634] = 332,
  [175635] = 332,
  [175636] = 332,
  [175637] = 332,
  [175638] = 332,
  [175639] = 332,
  [175640] = 332,
  [175641] = 332,
  [175642] = 332,
  [175643] = 332,
  [175644] = 332,
  [175645] = 332,
  [175646] = 332,
  [175647] = 332,
  [175648] = 332,
  [175649] = 332,
  [175650] = 332,
  [175651] = 332,
  [175652] = 332,
  [175653] = 332,
  [175654] = 332,
  [175655] = 332,
  [175656] = 332,
  [175657] = 332,
  [175658] = 332,
  [175659] = 332,
  [175660] = 332,
  [175661] = 332,
  [175662] = 332,
  [175663] = 332,
  [175664] = 332,
  [175665] = 332,
  [175666] = 332,
  [175667] = 332,
  [175668] = 332,
  [175669] = 332,
  [175670] = 332,
  [175671] = 332,
  [175672] = 332,
  [175673] = 332,
  [175674] = 332,
  [175675] = 332,
  [175676] = 332,
  [175677] = 332,
  [175678] = 332,
  [175679] = 332,
  [175680] = 332,
  [175681] = 332,
  [175682] = 332,
  [175683] = 332,
  [175684] = 332,
  [175685] = 332,
  [175686] = 332,
  [175687] = 332,
  [175688] = 332,
  [175689] = 332,
  [175690] = 332,
  [175691] = 332,
  [175692] = 332,
  [175693] = 332,
  [175694] = 332,
  [175710] = 317,
  [175711] = 317,
  [175712] = 317,
  [175713] = 317,
  [175714] = 317,
  [175715] = 317,
  [175716] = 317,
  [175717] = 317,
  [175718] = 317,
  [175719] = 317,
  [181495] = 54,
  [181496] = 54,
  [181497] = 54,
  [181498] = 54,
  [181499] = 54,
  [181500] = 54,
  [181501] = 54,
  [181502] = 54,
  [181503] = 54,
  [181504] = 54,
  [181505] = 54,
  [181506] = 54,
  [181507] = 54,
  [181508] = 54,
  [181511] = 54,
  [181512] = 54,
  [181513] = 54,
  [181514] = 54,
  [181515] = 54,
  [181516] = 54,
  [181517] = 54,
  [181518] = 54,
  [181519] = 54,
  [181520] = 54,
  [181521] = 54,
  [181522] = 54,
  [181523] = 54,
  [181524] = 54,
  [181525] = 54,
  [181526] = 54,
  [181527] = 54,
  [181528] = 54,
  [181529] = 54,
  [181530] = 54,
  [181531] = 54,
  [182221] = 716,
  [182222] = 716,
  [182223] = 716,
  [182224] = 716,
  [182225] = 716,
  [182226] = 716,
  [182227] = 716,
  [182228] = 716,
  [182229] = 716,
  [182231] = 715,
  [182232] = 715,
  [182233] = 715,
  [182234] = 715,
  [182235] = 715,
  [182236] = 715,
  [182237] = 715,
  [182238] = 715,
  [182239] = 715,
  [182240] = 715,
  [182241] = 715,
  [182242] = 715,
  [182243] = 715,
  [182244] = 715,
  [182245] = 715,
  [182246] = 715,
  [182247] = 715,
  [182248] = 715,
  [182249] = 715,
  [182250] = 715,
  [182251] = 715,
  [182252] = 715,
  [182253] = 715,
  [182254] = 715,
  [182255] = 715,
  [182256] = 715,
  [182257] = 715,
  [182259] = 714,
  [182260] = 714,
  [182261] = 714,
  [182262] = 714,
  [182263] = 714,
  [182264] = 714,
  [182265] = 714,
  [182266] = 714,
  [182267] = 714,
  [182268] = 714,
  [182269] = 714,
  [182270] = 714,
  [182271] = 714,
  [182272] = 714,
  [182273] = 714,
  [182274] = 714,
  [182275] = 714,
  [182276] = 714,
  [182277] = 714,
  [182278] = 714,
  [182279] = 714,
  [182287] = 707,
  [182296] = 699,
  [182297] = 699,
  [182298] = 699,
  [182299] = 699,
  [182300] = 699,
  [182301] = 699,
  [182314] = 687,
  [182316] = 686,
  [182612] = 391,
  [182920] = 84,
  [183202] = 1,
  [183852] = 1,
  [184075] = 54,
  [184076] = 54,
  [184077] = 54,
  [184078] = 54,
  [184079] = 54,
  [184080] = 54,
  [184081] = 54,
  [184082] = 54,
  [184083] = 54,
  [184084] = 54,
  [184085] = 54,
  [184086] = 54,
  [184087] = 54,
  [184088] = 54,
  [184089] = 54,
  [184090] = 54,
  [184091] = 54,
  [184092] = 54,
  [184093] = 54,
  [184094] = 54,
  [184095] = 54,
  [184096] = 54,
  [184097] = 54,
  [184098] = 54,
  [184099] = 54,
  [184100] = 54,
  [184101] = 54,
  [184102] = 54,
  [187797] = 395,
  [187816] = 329,
  [187817] = 329,
  [187818] = 329,
  [187819] = 329,
  [187820] = 329,
  [187821] = 329,
  [187822] = 329,
  [187823] = 329,
  [187824] = 329,
  [187825] = 329,
  [187826] = 329,
  [187827] = 329,
  [187828] = 329,
  [187829] = 329,
  [187830] = 329,
  [187831] = 329,
  [187832] = 329,
  [187833] = 329,
  [187834] = 329,
  [187835] = 329,
  [187836] = 329,
  [187837] = 329,
  [187838] = 329,
  [187839] = 329,
  [187840] = 329,
  [187841] = 329,
  [187842] = 329,
  [187843] = 329,
  [187844] = 329,
  [187845] = 329,
  [187846] = 329,
  [187847] = 329,
  [187848] = 329,
  [187849] = 329,
  [187850] = 329,
  [187851] = 329,
  [187852] = 329,
  [187853] = 329,
  [187854] = 329,
  [187855] = 329,
  [187857] = 328,
  [187858] = 328,
  [187859] = 328,
  [187862] = 326,
  [187863] = 326,
  [187864] = 326,
  [189466] = 608,
  [189467] = 608,
  [189468] = 608,
  [189469] = 608,
  [189470] = 608,
  [189471] = 608,
  [189472] = 608,
  [189483] = 598,
  [189484] = 598,
  [189485] = 598,
  [189486] = 598,
  [189487] = 598,
  [189488] = 598,
  [189489] = 598,
  [189490] = 598,
  [189491] = 598,
  [189492] = 598,
  [189493] = 598,
  [189494] = 598,
  [189495] = 598,
  [189496] = 598,
  [189497] = 598,
  [189498] = 598,
  [189499] = 598,
  [189500] = 598,
  [189501] = 598,
  [189502] = 598,
  [189503] = 598,
  [191159] = 1355,
  [191160] = 1355,
  [191161] = 1355,
  [191162] = 1355,
  [191163] = 1355,
  [191164] = 1355,
  [191165] = 1355,
  [191166] = 1355,
  [191167] = 1355,
  [191168] = 1355,
  [191169] = 1355,
  [191170] = 1355,
  [191171] = 1355,
  [191172] = 1355,
  [191173] = 1355,
  [191174] = 1355,
  [191175] = 1355,
  [191176] = 1355,
  [191177] = 1355,
  [191178] = 1355,
  [191179] = 1355,
  [191180] = 1355,
  [191181] = 1355,
  [191182] = 1355,
  [191183] = 1355,
  [191184] = 1355,
  [191185] = 1355,
  [191186] = 1355,
  [191187] = 1355,
  [191188] = 1355,
  [191189] = 1355,
  [191190] = 1355,
  [191191] = 1355,
  [191192] = 1355,
  [191193] = 1355,
  [191194] = 1355,
  [191195] = 1355,
  [191196] = 1355,
  [191197] = 1355,
  [191198] = 1355,
  [191199] = 1355,
  [191200] = 1355,
  [191201] = 1355,
  [191202] = 1355,
  [191203] = 1355,
  [191204] = 1355,
  [191205] = 1355,
  [191206] = 1355,
  [191207] = 1355,
  [191208] = 1355,
}

-- Essence Rune Jaedi |H1:item:45844:20:1:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h - Kaderi
-- Essence Rune Lire |H1:item:45845:20:1:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h - Taderi
function TTCCompanion:BuildRemovedItemIdTable()
  local removedItemId = { 45844, 45845 }
  for _, itemId in pairs(removedItemId) do
    TTCCompanion.removedItemIdTable[itemId] = true
  end
end

function TTCCompanion:GetItemLinkFromItemId(itemId)
  return string.format("|H1:item:%d:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h", itemId)
end

local function indexRecipes(t, index)
  local itemId = GetItemLinkItemId(index)
  local offset = TTCCompanion.internalRecipeData[itemId]
  if offset == nil then
    return nil
  end
  local recipeId = itemId + offset

  if recipeId then
    -- Item is in the list!
    return TTCCompanion:GetItemLinkFromItemId(recipeId)
  end
  return nil
end

local recipeMetatable = {
  ["__index"] = indexRecipes,
}
TTCCompanion.recipeData = {}
setmetatable(TTCCompanion.recipeData, recipeMetatable)

-- 1 == positive potency 2 == negative potency
-- 1 == Additive, 2 == Subtractive
local effectIds = {
  [43573] = { 45831, 2 },
  [26580] = { 45831, 1 },
  [45868] = { 45832, 2 },
  [26582] = { 45832, 1 },
  [45867] = { 45833, 2 },
  [26588] = { 45833, 1 },
  [26581] = { 45834, 1 },
  [45869] = { 45834, 2 },
  [26583] = { 45835, 1 },
  [45870] = { 45835, 2 },
  [26589] = { 45836, 1 },
  [45871] = { 45836, 2 },
  [26586] = { 45837, 2 },
  [26587] = { 45837, 1 },
  [26848] = { 45838, 1 },
  [26849] = { 45838, 2 },
  [5365] = { 45839, 1 },
  [5364] = { 45839, 2 },
  [43570] = { 45840, 2 },
  [26844] = { 45840, 1 },
  [26841] = { 45841, 1 },
  [26847] = { 45841, 2 },
  [26845] = { 45842, 2 },
  [5366] = { 45842, 1 },
  [54484] = { 45843, 1 },
  [26591] = { 45843, 2 },
  [45874] = { 45846, 1 },
  [45875] = { 45846, 2 },
  [45885] = { 45847, 2 },
  [45883] = { 45847, 1 },
  [45886] = { 45848, 2 },
  [45884] = { 45848, 1 },
  [45873] = { 45849, 2 },
  [45872] = { 45849, 1 },
  [166046] = { 166045, 2 },
  [166047] = { 166045, 1 },
  [68344] = { 68342, 2 },
  [68343] = { 68342, 1 },
}

local potencyIds = {
  [5] = { 45855, 45817 },
  [10] = { 45856, 45818 },
  [15] = { 45857, 45819 },
  [20] = { 45806, 45820 },
  [25] = { 45807, 45821 },
  [30] = { 45808, 45822 },
  [35] = { 45809, 45823 },
  [40] = { 45810, 45824 },
  [45] = { 45811, 45825 },
  [51] = { 45812, 45826 },
  [53] = { 45813, 45827 },
  [55] = { 45814, 45828 },
  [57] = { 45815, 45829 },
  [63] = { 45816, 45830 },
  [65] = { 64509, 64508 },
  [66] = { 68341, 68340 },
}

-- aspect rune ID numbers
local qualityIds = {
  [1] = 45850,
  [2] = 45851,
  [3] = 45852,
  [4] = 45853,
  [5] = 45854,
}

local function splitItem(t, index)
  local itemId, lvl, CPLVL, quality = zo_strmatch(index, "(%d+):(%d+):(%d+):(%d+)")
  itemId = tonumber(itemId)
  lvl = tonumber(lvl)
  CPLVL = tonumber(CPLVL)
  quality = tonumber(quality)
  local combinedLvl = lvl + CPLVL
  local essence = effectIds[itemId]
  if not essence then return end
  if not potencyIds[combinedLvl] then return end
  local potency = potencyIds[combinedLvl][essence[2]]
  local quality = qualityIds[quality]
  if not potency or not quality or not essence then return end
  return {
    { ["item"] = TTCCompanion:GetItemLinkFromItemId(potency), ["required"] = 1 },
    { ["item"] = TTCCompanion:GetItemLinkFromItemId(essence[1]), ["required"] = 1 },
    { ["item"] = TTCCompanion:GetItemLinkFromItemId(quality), ["required"] = 1 }
  }
end

TTCCompanion.virtualRecipe = {}
setmetatable(TTCCompanion.virtualRecipe, { ["__index"] = splitItem })
